local addonName, addonTable = ...
local hbd = LibStub("HereBeDragons-2.0")
local TIMER_LENGTH = 0.25
local BASE_WORLDMAP_ICON_SIZE = 8
local worldMapIconSize = 2

local recycledPins = {}
local minimapPins = {}
local worldMapPins = {}
local numberOfPins = 0
local showWorldMapIcons = false
local showMinimapIcons = true
local onlyShowFactionQuests = true
local playerFaction = nil
local questLevelMaxRange = 0
local playerLevel = 0
local shownIconTypes = {}

function addonTable:GetDistanceFromPlayer(itemX, itemY, playerLocation)
    local xDist = math.abs(itemX - playerLocation.x)
    local yDist = math.abs(itemY - playerLocation.y)

    local distance = math.sqrt((xDist * xDist) + (yDist * yDist))
    return distance
end

function addonTable:GetAlphaValueFromDistance(distance)
    local zoomLevel = Minimap:GetZoom()
    local zoomDistance = addonTable[zoomLevel]
    if distance > addonTable.zoomLevelDistances[zoomLevel] then
        return 0
    end

    return 1
end

function addonTable:GetPlayerLocation()
    local mapID = addonTable:GetCurrentZone()
    local playerLocation = C_Map.GetPlayerMapPosition(mapID, "player");
    return playerLocation
end 

function addonTable:MinimapIconUpdate(elapsed)
    if(self.IsMinimapIcon) then
        self.TimeSinceUpdate = self.TimeSinceUpdate + elapsed;

        if(self.TimeSinceUpdate > TIMER_LENGTH) then
            self.TimeSinceUpdate = 0

            if(self.isShown == false) then
                self.texture:SetVertexColor(1, 1, 1, 0)
                return
            end
            
            if(addonTable:ShouldRenderIcon(self.Name, self.factionType, self.MinLevel) == false) then
                self.texture:SetVertexColor(1, 1, 1, 0)
                return
            end

            local playerLocation = addonTable:GetPlayerLocation()
            local uiMapID = addonTable:GetCurrentZone()
            local distance = hbd:GetZoneDistance(uiMapID, self.ItemLocation[1], self.ItemLocation[2], uiMapID, playerLocation.x, playerLocation.y)
            local alphaValue = addonTable:GetAlphaValueFromDistance(distance)        
            self.texture:SetVertexColor(1, 1, 1, alphaValue)
        end
    end
end

function addonTable:WorldMapIconUpdate(elapsed)
    if(self.IsMinimapIcon == false) then
        self.TimeSinceUpdate = self.TimeSinceUpdate + elapsed;

        if(self.TimeSinceUpdate > TIMER_LENGTH) then
            self.TimeSinceUpdate = 0
            if self.isShown == false then
                self.texture:SetVertexColor(1, 1, 1, 0)
            else
                if addonTable:ShouldRenderIcon(self.Name, self.factionType, self.MinLevel) then
                    self.texture:SetVertexColor(1, 1, 1, 1)
                else
                    self.texture:SetVertexColor(1, 1, 1, 0)
                end
            end
        end
    end
end

function addonTable:ShouldRenderIcon(itemType, factionType, level)
    return (addonTable:ShouldBeShowingItemType(itemType)and
    addonTable:ShouldBeShowingFactionType(factionType) and
    addonTable:ShouldBeRenderingQuest(itemType, level))
end

function addonTable:ShouldBeShowingFactionType(factionType)
    if (factionType == nil) then
        return true
    end

    if(onlyShowFactionQuests == true) then
        return factionType == playerFaction
    else
        return true
    end
end

function addonTable:ShouldBeShowingItemType(itemType)
    if(shownIconTypes[itemType] == nil) then
        return false
    else
        return true
    end
end

function addonTable:ShouldBeRenderingQuest(itemType, level)
    if(itemType == nil) then
        return true
    end

    if(level == nil) then
        return true
    end

    local temp = playerLevel + questLevelMaxRange
    if(temp >= level) then
        return true
    else
        return false
    end
end

local function CreateNewFrame(itemCoords, playerLocation)
    local newFrame = next(recycledPins)
    if newFrame then
        if(itemCoords[3] == "Quest") then
            newFrame.factionType = itemCoords[4]
            newFrame.MinLevel = itemCoords[5]
        end
        recycledPins[newFrame] = nil
        return newFrame
    else
        numberOfPins = numberOfPins + 1
        newFrame = CreateFrame("Frame","ItemFrame"..numberOfPins,UIParent)
        newFrame.IconID = numberOfPins
    end

    newFrame:SetFrameStrata("BACKGROUND")

    if(itemCoords[3] == "Quest") then
        newFrame.factionType = itemCoords[4]
        newFrame.MinLevel = itemCoords[5]
    end

    return newFrame
end

function addonTable:CreateMinimapPin(itemCoords, playerLocation)
    local newFrame = CreateNewFrame(itemCoords, playerLocation)
    
    local itemLocation = {itemCoords[1], itemCoords[2]}
    newFrame.ItemLocation = itemLocation
    newFrame.TimeSinceUpdate = 0
    newFrame.IsMinimapIcon = true
    newFrame.Name = itemCoords[3]
    newFrame:SetScript("OnUpdate", addonTable.MinimapIconUpdate)

    newFrame:SetWidth(16)
    newFrame:SetHeight(16)
    local newTexture = newFrame:CreateTexture(nil,"BACKGROUND")
    local iconName = addonTable.itemNameLookups[itemCoords[3]]
    local iconLookup = addonTable.iconLookups[iconName]
    newTexture:SetTexture(iconLookup)
    newTexture:SetAllPoints(newFrame)
    newFrame.texture = newTexture
    newFrame:SetPoint("CENTER",0,0)
    newFrame.texture:SetVertexColor(1, 1, 1, 0)

    minimapPins[newFrame] = newFrame
    return newFrame
end

function addonTable:CreateWorldMapPin(itemCoords, playerLocation)
    local newFrame = CreateNewFrame(itemCoords, playerLocation)
    newFrame.isShown = showWorldMapIcons
    newFrame.TimeSinceUpdate = 0
    newFrame.Name = itemCoords[3]
    newFrame:SetScript("OnEnter", addonTable.ShowIconTooltip)
    newFrame:SetScript("OnLeave", addonTable.HideIconTooltip)
    newFrame:SetScript("OnUpdate", addonTable.WorldMapIconUpdate)
    newFrame.IsMinimapIcon = false

    newFrame:SetWidth(8 * worldMapIconSize)
    newFrame:SetHeight(8 * worldMapIconSize)
    local newTexture = newFrame:CreateTexture(nil,"BACKGROUND")
    local iconName = addonTable.itemNameLookups[itemCoords[3]]
    local iconLookup = addonTable.iconLookups[iconName]
    newTexture:SetTexture(iconLookup)
    newTexture:SetAllPoints(newFrame)
    newFrame.texture = newTexture
    newFrame:SetPoint("CENTER",0,0)
    newFrame.texture:SetVertexColor(1, 1, 1, 0)
    worldMapPins[newFrame] = newFrame

    return newFrame
end

function addonTable:ToggleWorldMapIcons()
    if(showWorldMapIcons == true) then
        showWorldMapIcons = false
    else
        showWorldMapIcons = true
    end

    for key, frame in pairs(worldMapPins) do
        frame.isShown = showWorldMapIcons
    end

end

function addonTable:ShowHideWorldMapIcons(showIcons)
    showWorldMapIcons = showIcons

    for key, frame in pairs(worldMapPins) do
        frame.isShown = showWorldMapIcons
    end
end

function addonTable:ToggleMinimapIcons(showIcons)
    showMinimapIcons = showIcons

    for key, frame in pairs(minimapPins) do
        frame.isShown = showMinimapIcons
    end
end

function addonTable:FilterWorldMapIcons(itemType, showType)
    if(showType) then
        if(shownIconTypes[itemType] == nil) then
            shownIconTypes[itemType] = true
        end
    else
        shownIconTypes[itemType] = nil
    end
end

function addonTable:GetMinimapIconsStatus()
    return showMinimapIcons
end

function addonTable:GetWorldMapIconStatus(value)
    if(shownIconTypes[value] == nil) then
        return false
    else
        return true
    end
end

function addonTable:GetWorldMapIconsStatus()
    return showWorldMapIcons
end

function addonTable:ToggleAllWorldMapIcons(iconTypes, value)
    for iconType, _ in pairs(iconTypes) do
        if(shownIconTypes[iconType] == nil) then
            if(value) then
                shownIconTypes[iconType] = true
            end
        else
            if(value == false) then
                shownIconTypes[iconType] = nil
            end
        end
    end
end

function addonTable:UpdateWorldMapIconSize(value)
    worldMapIconSize = value

    for key, frame in pairs(worldMapPins) do
        frame:SetWidth(8 * worldMapIconSize)
        frame:SetHeight(8 * worldMapIconSize)
    end
end

function addonTable:GetWorldMapIconSize()
    return worldMapIconSize
end

function addonTable:ShowIconTooltip()
    if(self.isShown) then
        if addonTable:ShouldRenderIcon(self.Name, self.factionType, self.MinLevel) then
            self:SetWidth(8 * worldMapIconSize + 16)
            self:SetHeight(8 * worldMapIconSize + 16)
            local tooltip = GameTooltip
            tooltip._owner = self
            tooltip:SetOwner(self, "ANCHOR_BOTTOM")
            tooltip:SetFrameStrata("TOOLTIP")
            local text = self.Name
            tooltip._rebuild = function(self)
                self:AddLine(text)
            end
            tooltip:_rebuild()
            tooltip:Show()
        end
    end
end

function addonTable:HideIconTooltip()
    self:SetWidth(8 * worldMapIconSize)
    self:SetHeight(8 * worldMapIconSize)
    if(GameTooltip) then
        GameTooltip:Hide()
        GameTooltip._rebuild = nil
    end
end

function addonTable:OnlyShowFactionQuests()
    return onlyShowFactionQuests
end

function addonTable:ToggleOnlyFactionQuests(value)
    onlyShowFactionQuests = value
end

function addonTable:SetPlayerFaction(factionName)
    if(factionName == "Alliance") then
        playerFaction = "a"
    else
        playerFaction = "h"
    end
end

function addonTable:QuestLevelMaxRange()
    return questLevelMaxRange
end

function addonTable:SetQuestLevelMaxRange(value)
    questLevelMaxRange = value
end

function addonTable:SetPlayerLevel(level)
    playerLevel = level
end

function addonTable:ClearAllPins()
    for key, frame in pairs(minimapPins) do
        frame.texture:Hide()
        frame:SetParent(nil)
        frame.texture:ClearAllPoints()
        frame:SetScript("OnUpdate", nil)
        frame:SetScript("OnEnter", nil)
        frame:SetScript("OnLeave", nil)
        frame:UnregisterAllEvents()
        recycledPins[frame] = true
        minimapPins[frame] = nil
        frame.factionType = nil
        frame.MinLevel = nil
        frame.ItemLocation = nil
        frame.TimeSinceUpdate = nil
        frame.IsMinimapIcon = nil
        frame.Name = nil
    end

    minimapPins = {}

    for key, frame in pairs(worldMapPins) do
        frame.texture:Hide()
        frame:SetParent(nil)
        frame.texture:ClearAllPoints()
        frame:SetScript("OnUpdate", nil)
        frame:SetScript("OnEnter", nil)
        frame:SetScript("OnLeave", nil)
        frame:UnregisterAllEvents()
        recycledPins[frame] = true
        worldMapPins[frame] = nil
        frame.factionType = nil
        frame.MinLevel = nil
        frame.isShown = nil
        frame.TimeSinceUpdate = nil
        frame.Name = nil
    end

    worldMapPins = {}
end

function addonTable:GetNumberOfPins()
    local count = 0
    for key, frame in pairs(minimapPins) do
        count = count + 1
    end
    print("There are ", count, " in minimappins")

    count = 0
    for key, frame in pairs(recycledPins) do
        count = count + 1
    end
    print("There are ", count, " in recycledPins")
end