local title, sharedTable = ...

GamerMapsAddon = LibStub("AceAddon-3.0"):NewAddon("GamerMapsAddon", "AceConsole-3.0", "AceEvent-3.0")
local hbd = LibStub("HereBeDragons-2.0")
local hbdp = LibStub("HereBeDragons-Pins-2.0")

local options = {
    name = "GamerMapsAddon",
    handler = GamerMapsAddon,
    type = 'group',
        args = {
        ver = {
            type = "input",
            name = "Version",
            desc = "Gets the current Item DB Version.",
            set = "GetVersion",
        },
    },
}
local allWorldmapIconsEnabled = false

function GamerMapsAddon:OnInitialize()
    LibStub("AceConfig-3.0"):RegisterOptionsTable("GamerMapsAddon", options, {"GamerMapsAddon", "GM"})
end

function GamerMapsAddon:OnEnable()
    C_Timer.After(1, function()
        GamerMapsAddon:Start()
    end)
end

function GamerMapsAddon:Start()
    local englishFaction, localizedFaction = UnitFactionGroup("player")
    sharedTable:SetPlayerFaction(englishFaction)
    sharedTable:SetPlayerLevel(UnitLevel("player"))

    iconTypes = {}
    sharedTable:InitalizeItemTypes()
    for index, itemType in ipairs(sharedTable:GetDistinctItemTypes()) do
        iconTypes[itemType] = itemType
        GamerMapsAddon:ToggleWorldMapIconType(itemType, true)
        GamerMapsAddon:ToggleAllWorldMapIcons(true)
    end

    local addonOptions = {
        name = "GamerMaps",
        handler = GamerMapsAddon,
        type = "group",
        childGroups = "tab",
        args = {
            options_tab = {
                name ="Options",
                type = "group",
                order = 10,
                args = {
                    minimap_icons_header = {
                        name = "Minimap Icons Options",
                        type = "header",
                        order = 1,
                    },
                    minimap_icons_enabled = {
                        type = "toggle",
                        order = 2,
                        name = "Show Icons on Minimap",
                        desc = "Used to turn ALL icons ON or OFF on the Minimap",
                        width = "full",
                        get = function()
                                return sharedTable:GetMinimapIconsStatus()
                        end,
                        set = function(info, value)
                                sharedTable:ToggleMinimapIcons(value)
                        end,
                    },
                    world_icons_header = {
                        name = "Worldmap Icons Options",
                        type = "header",
                        order = 3,
                    },
                    worldmap_icons_enabled = {
                        type = "toggle",
                        order = 4,
                        name = "Show Icons on Worldmap",
                        desc = "Used to turn ALL icons ON or OFF on the Worldmap",
                        width = "full",
                        get = function()
                                return sharedTable:GetWorldMapIconsStatus()
                        end,
                        set = function(info, value)
                            GamerMapsAddon:ToggleWorldMapIcons(value)
                        end,
                    },
                    world_maps_icon_size = {
                        type = "range",
                        order = 5,
                        name = "WorldMap Icons size",
                        desc = "Sets the size of the Icons on the Worldmap",
                        width = "normal",
                        min = 1,
                        max = 5,
                        step = 1,
                        get = function()
                                return sharedTable:GetWorldMapIconSize()
                        end,
                        set = function(info, value)
                                GamerMapsAddon:SetWorldMapIconSize(value)
                        end,
                    },
                    quest_options_header = {
                        name = "Quest Options",
                        desc = "Options for rendering Quest Icons",
                        type = "header",
                        order = 6,
                    },
                    enable_only_faction_quests = {
                        name = "Only Show Quests For My Faction",
                        type = "toggle",
                        order = 7,
                        width = "full",
                        get = function(info, value)
                                return sharedTable:OnlyShowFactionQuests()
                        end,
                        set = function(info, value)
                                sharedTable:ToggleOnlyFactionQuests(value)
                        end,
                    },
                    quest_level_range = {
                        type = "range",
                        order = 8,
                        name = "Quest Level Range",
                        desc = "Used to show Quests with a Minimum Level this many Levels above your current Level",
                        width = "normal",
                        min = 0,
                        max = 15,
                        step = 1,
                        get = function()
                                return sharedTable:QuestLevelMaxRange()
                        end,
                        set = function(info, value)
                                sharedTable:SetQuestLevelMaxRange(value)
                        end,
                    },
                    icon_types_header = {
                        name = "Icon Types to Show on Maps",
                        desc = "Used to toggle certain Icon Types on or off on BOTH maps",
                        type = "header",
                        order = 9,
                    },
                    enable_all_world_map_icons = {
                        name = "Enable All Icon Types",
                        type = "toggle",
                        order = 10,
                        width = "full",
                        get = function(info, value)
                                return allWorldmapIconsEnabled
                        end,
                        set = function(info, value)
                                GamerMapsAddon:ToggleAllWorldMapIcons(value)
                        end,
                    },
                    icon_types_select = {
                        name = "Icon Types",
                        type = "multiselect",
                        order = 11,
                        values = iconTypes,
                        get = function(info, value)
                                return sharedTable:GetWorldMapIconStatus(value)
                        end,
                        set = function(info, value, state)
                                GamerMapsAddon:ToggleWorldMapIconType(value, state)
                        end,
                    }
                }
            }                     
        }
    }

    LibStub("AceConfig-3.0"):RegisterOptionsTable("GamerMaps", addonOptions)
    self.configFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GamerMaps", "GamerMaps");

    self:RegisterEvent("ZONE_CHANGED")
    sharedTable:InitZoneLookups()
    local mapID, areaID = sharedTable:GetMapAndAreaID()
    GamerMapsAddon:AddItemsToMaps(mapID, areaID)
end

function GamerMapsAddon:OnDisable()
end

function GamerMapsAddon:ZONE_CHANGED()
    hbdp:RemoveAllWorldMapIcons("GamerMapsAddon")
    sharedTable:ClearAllPins()
    hbdp:RemoveAllMinimapIcons("GamerMapsAddon")
    local mapID, areaID = sharedTable:GetMapAndAreaID()
    GamerMapsAddon:AddItemsToMaps(mapID,areaID)
end

function GamerMapsAddon:HideWorldMapIcons()
    sharedTable:ToggleWorldMapIcons()
end

function GamerMapsAddon:SetWorldMapIconSize(value)
    sharedTable:UpdateWorldMapIconSize(value)
end

function GamerMapsAddon:ToggleWorldMapIcons(value)
    sharedTable:ShowHideWorldMapIcons(value)
end

function GamerMapsAddon:ToggleWorldMapIconType(itemType, state)
    sharedTable:FilterWorldMapIcons(itemType, state)
end

function GamerMapsAddon:ToggleAllWorldMapIcons(value)
    allWorldmapIconsEnabled = value
    sharedTable:ToggleAllWorldMapIcons(iconTypes, value)
end

function GamerMapsAddon:AddItemsToMaps(mapID, areaID)
    local playerLocation = C_Map.GetPlayerMapPosition(mapID, "player")

    if(sharedTable.items[areaID] == nil) then
        return
    end

    for index, itemCoords in ipairs(sharedTable.items[areaID]) do

        if(itemCoords[1] > 1) then
            local newX = itemCoords[1] / 100.0 / 2.94345
            local newY = itemCoords[2] / 100.0 / 2.94345
            itemCoords[1] = newX
            itemCoords[2] = newY
        end

        local newMinimapPin = sharedTable:CreateMinimapPin(itemCoords, playerLocation)
        local newWorldMapPin = sharedTable:CreateWorldMapPin(itemCoords, playerLocation)      
        hbdp:AddMinimapIconMap("GamerMapsAddon", newMinimapPin, mapID, itemCoords[1], itemCoords[2], true, true)
        hbdp:AddWorldMapIconMap("GamerMapsAddon", newWorldMapPin, mapID, itemCoords[1], itemCoords[2], 1)
    end
end

function GamerMapsAddon:SetMarker(info, newValue)
    local mapID = C_Map.GetBestMapForUnit("player")
    local coords = C_Map.GetPlayerMapPosition(mapID, "player")
    local mapInfo = C_Map.GetMapInfo(mapID)

    local f = CreateFrame("Frame",nil,UIParent)
    f:SetFrameStrata("BACKGROUND")
    f:SetWidth(16)
    f:SetHeight(16)
    local t = f:CreateTexture(nil,"BACKGROUND")
    t:SetTexture("Interface\\Addons\\GamerMapsAddon\\slay.blp")
    t:SetAllPoints(f)
    f.texture = t
    f.ItemLocation = {coords.x, coords.y}
    f.TimeSinceUpdate = 0
    f:SetPoint("CENTER",0,0)

    f:SetScript("OnUpdate", sharedTable.MinimapIconUpdate)

    hbdp:AddMinimapIconMap("GamerMapsAddon", f, mapID, coords.x, coords.y, true, true)
    table.insert(minimapPins, f)  
end

function GamerMapsAddon:GetVersion()
    print("Version is: ", sharedTable.itemsVersion)
end